#pragma once

#include <map>

#include "HEJ/Event.hh"
#include "HEJ/event_types.hh"

namespace HEJFOG {
  template<typename T>
  struct WithError {
    T value = T{};
    T error = T{};
  };

  class CrossSectionAccumulator {
  public:
    void fill(HEJ::Event const & ev) {
      const double wt = ev.central().weight;
      auto & entry = xs_[ev.type()];
      entry.value += wt;
      entry.error += wt*wt;
      total_.value += wt;
      total_.error += wt*wt;
    }

    auto begin() const {
      return std::begin(xs_);
    }

    auto end() const {
      return std::end(xs_);
    }

    WithError<double> total() const {
      return total_;
    }

  private:
    std::map<HEJ::event_type::EventType, WithError<double>> xs_;
    WithError<double> total_;
  };

}
