#include "Unweighter.hh"

#include <cassert>

#include "HEJ/Event.hh"

namespace HEJFOG {

  namespace detail {
    bool has_jet_softer_than(HEJ::Event const & ev, double pt) {
      assert(! ev.jets().empty());
      const auto softest_jet = fastjet::sorted_by_pt(ev.jets()).back();
      return softest_jet.pt() < pt;
    }
  }

  namespace {
    void normalise_weights(HEJ::Event & ev, double target) {
      const double awt = std::abs(ev.central().weight);
      ev.central().weight *= target/awt;
      for(auto & var: ev.variations()) var.weight *= target/awt;
    }
  }

  HEJ::optional<HEJ::Event> Unweighter::unweight(HEJ::Event ev) const {
    if(detail::has_jet_softer_than(ev, min_unweight_pt_)) return ev;
    const double awt = std::abs(ev.central().weight);
    if(ran_.get().flat() < awt/cut_) {
      if(awt < cut_) normalise_weights(ev, cut_);
      return ev;
    }
    return {};
  }
}
