#ifdef NDEBUG
#undef NDEBUG
#endif

#include <algorithm>
#include <cmath>
#include <cassert>
#include <iostream>

#include "config.hh"
#include "EventGenerator.hh"
#include "HEJ/Mixmax.hh"

#include "HEJ/Event.hh"
#include "HEJ/PDF.hh"
#include "HEJ/MatrixElement.hh"

using namespace HEJFOG;

int main(){
  constexpr double invGeV2_to_pb = 389379292.;
  constexpr double xs_ref = 86.42031848*1e6; //calculated with "combined" HEJ svn r3480

  auto config = load_config("config_2j.yml");

  HEJ::Mixmax ran{};
  HEJFOG::EventGenerator generator{
    config.process,
    config.beam,
    HEJ::ScaleGenerator{
      config.scales.base,
      config.scales.factors,
      config.scales.max_ratio
    },
    config.jets,
    config.pdf_id,
    config.subleading_fraction,
    config.subleading_channels,
    config.particles_properties,
    config.Higgs_coupling,
    ran
  };

  double xs = 0., xs_err = 0.;
  for (int trials = 0; trials < config.events; ++trials){
    auto ev = generator.gen_event();
    if(generator.status() != good) continue;
    ev.central().weight *= invGeV2_to_pb;
    ev.central().weight /= config.events;

    xs += ev.central().weight;
    xs_err += ev.central().weight*ev.central().weight;
  }
  xs_err = std::sqrt(xs_err);
  std::cout << xs_ref << " ~ " << xs << " +- " << xs_err << '\n';

  assert(std::abs(xs - xs_ref) < 3*xs_err);
  assert(xs_err < 0.01*xs);
}
