#ifdef NDEBUG
#undef NDEBUG
#endif

#include <algorithm>
#include <cmath>
#include <cassert>
#include <iostream>

#include "config.hh"
#include "EventGenerator.hh"
#include "HEJ/Ranlux64.hh"

#include "HEJ/Event.hh"
#include "HEJ/MatrixElement.hh"
#include "HEJ/PDF.hh"

using namespace HEJFOG;

int main(){
  constexpr double invGeV2_to_pb = 389379292.;
  constexpr double xs_ref = 1.07807; // +- 0.0071
  //calculated with HEJ revision 93efdc851b02a907a6fcc63956387f9f4c1111c2 +1

  auto config = load_config("config_h_2j.yml");
  config.process.njets = 3;

  HEJ::Ranlux64 ran{};
  HEJFOG::EventGenerator generator{
    config.process,
    config.beam,
    HEJ::ScaleGenerator{
      config.scales.base,
      config.scales.factors,
      config.scales.max_ratio
    },
    config.jets,
    config.pdf_id,
    config.subleading_fraction,
    config.subleading_channels,
    config.particles_properties,
    config.Higgs_coupling,
    ran
  };

  double xs = 0., xs_err = 0.;
  for (int trials = 0; trials < config.events; ++trials){
    auto ev = generator.gen_event();
    if(generator.status() != good) continue;
    ev.central().weight *= invGeV2_to_pb;
    ev.central().weight /= config.events;

    const auto the_Higgs = std::find_if(
        begin(ev.outgoing()), end(ev.outgoing()),
        [](HEJ::Particle const & p){ return p.type == HEJ::ParticleID::h; }
    );
    assert(the_Higgs != end(ev.outgoing()));
    if(std::abs(the_Higgs->rapidity()) > 5.) continue;

    xs += ev.central().weight;
    xs_err += ev.central().weight*ev.central().weight;
  }
  xs_err = std::sqrt(xs_err);
  std::cout << xs_ref << " ~ " << xs << " +- " << xs_err << std::endl;

  assert(std::abs(xs - xs_ref) < 3*xs_err);
  assert(xs_err < 0.02*xs);
}
