#ifdef NDEBUG
#undef NDEBUG
#endif

// check that adding uno emissions doesn't change the FKL cross section

#include <algorithm>
#include <cassert>
#include <cmath>
#include <iostream>

#include "config.hh"
#include "EventGenerator.hh"
#include "HEJ/Ranlux64.hh"
#include "Subleading.hh"

#include "HEJ/Event.hh"
#include "HEJ/MatrixElement.hh"
#include "HEJ/PDF.hh"

using namespace HEJFOG;

int main(){
  constexpr double invGeV2_to_pb = 389379292.;
  constexpr double xs_ref = 0.0243548; // +- 0.000119862
  //calculated with HEJ revision 9570e3809613272ac4b8bf3236279ba23cf64d20

  auto config = load_config("config_h_2j.yml");
  config.process.njets = 3;
  config.process.incoming = {HEJ::pid::u, HEJ::pid::u};
  config.subleading_channels = HEJFOG::Subleading::uno;

  HEJ::Ranlux64 ran{};
  HEJFOG::EventGenerator generator{
    config.process,
    config.beam,
    HEJ::ScaleGenerator{
      config.scales.base,
      config.scales.factors,
      config.scales.max_ratio
    },
    config.jets,
    config.pdf_id,
    config.subleading_fraction,
    config.subleading_channels,
    config.particles_properties,
    config.Higgs_coupling,
    ran
  };

  double xs = 0., xs_err = 0.;
  int uno_found = 0;
  for (int trials = 0; trials < config.events; ++trials){
    auto ev = generator.gen_event();
    if(generator.status() != good) continue;
    if(ev.type() != HEJ::event_type::FKL){
      ++uno_found;
      continue;
    }
    ev.central().weight *= invGeV2_to_pb;
    ev.central().weight /= config.events;

    xs += ev.central().weight;
    xs_err += ev.central().weight*ev.central().weight;
  }
  xs_err = std::sqrt(xs_err);
  std::cout << xs_ref << " ~ " << xs << " +- " << xs_err << '\n';
  std::cout << uno_found << " events with unordered emission" << std::endl;
  assert(uno_found > 0);
  assert(std::abs(xs - xs_ref) < 3*xs_err);
  assert(xs_err < 0.05*xs);
}
