/** \file
 *  \brief Contains the EventWriter for HepMC Output.
 *
 *  \authors   Jeppe Andersen, Tuomas Hapola, Marian Heil, Andreas Maier, Jennifer Smillie
 *  \date      2019
 *  \copyright GPLv2 or later
 */

#pragma once

#include <memory>
#include <string>

#include "HEJ/EventWriter.hh"

namespace LHEF {
  class HEPRUP;
}

namespace HEJ{
  class Event;

  //! This is an event writer specifically for HepMC output.
  /**
   * \internal Implementation note:
   * This uses the pimpl ("pointer to implementation") idiom.
   * HepMC support is optional and the implementation depends on the
   * HepMC version. Without pimpl, we would have to specify the HepMC version
   * via the preprocessor whenever this header is included. We don't want to
   * burden users of the HEJ library (for example the HEJ fixed-order generator)
   * with those details
   */
  class HepMCWriter: public EventWriter{
  public:
    //! Constructor
    /**
     * @param file      name of the output file
     * @param heprup    general process information
     */
    HepMCWriter(std::string const & file, LHEF::HEPRUP heprup);
    ~HepMCWriter() override = default;

    //! Write an event to the output file
    void write(Event const & ev) override;

  private:
    struct HepMCWriterImpl;
    struct HepMCWriterImplDeleter {
      void operator()(HepMCWriterImpl* p);
    };

    std::unique_ptr<HepMCWriterImpl, HepMCWriterImplDeleter> impl_;
  };

}
