/** \file
 *  \brief Defines formats for output to event files
 *
 *  \authors   Jeppe Andersen, Tuomas Hapola, Marian Heil, Andreas Maier, Jennifer Smillie
 *  \date      2019
 *  \copyright GPLv2 or later
 */

#pragma once

#include <stdexcept>
#include <string>

namespace HEJ{

  //! Supported event file formats
  enum FileFormat{
    Les_Houches, /*!< Les Houches Output */
    HepMC        /*!< HepMC Output */
  };

  //! Convert a file format to a string
  inline std::string to_string(FileFormat f){
    switch(f){
    case Les_Houches: return "Les Houches";
    case HepMC: return "HepMC";
    default:
      throw std::logic_error("unhandled file format");
    }
  }

  //! Output file specification
  struct OutputFile{
    std::string name;     /**< Output File Name */
    FileFormat format;    /**< Output File Format */
  };

}
