/**
 *  \authors   Jeppe Andersen, Tuomas Hapola, Marian Heil, Andreas Maier, Jennifer Smillie
 *  \date      2019
 *  \copyright GPLv2 or later
 */
#include "HEJ/Weights.hh"

#include <stdexcept>

namespace HEJ {

  Weights& Weights::operator*=(Weights const & other) {
    if(other.variations.size() != variations.size()) {
      throw std::invalid_argument{"Wrong number of weights"};
    }
    central *= other.central;
    for(std::size_t i = 0; i < variations.size(); ++i) {
      variations[i] *= other.variations[i];
    }
    return *this;
  }

  Weights& Weights::operator*=(double factor) {
    central *= factor;
    for(auto & wt: variations) wt *= factor;
    return *this;
  }

  Weights& Weights::operator/=(Weights const & other) {
    if(other.variations.size() != variations.size()) {
      throw std::invalid_argument{"Wrong number of weights"};
    }
    central /= other.central;
    for(std::size_t i = 0; i < variations.size(); ++i) {
      variations[i] /= other.variations[i];
    }
    return *this;
  }

  Weights& Weights::operator/=(double factor) {
    central /= factor;
    for(auto & wt: variations) wt /= factor;
    return *this;
  }

}
