#include "LHEF/LHEF.h"
#include "HEJ/stream.hh"
#include "HEJ/config.hh"
#include "HEJ/event_types.hh"
#include "HEJ/Event.hh"
#include "HEJ/PhaseSpacePoint.hh"
#include "HEJ/Ranlux64.hh"

namespace{
  constexpr int max_trials = 100;
  constexpr double extpartonptmin = 45.;
  constexpr double max_ext_soft_pt_fraction =
    std::numeric_limits<double>::infinity();
  const fastjet::JetDefinition jet_def{fastjet::kt_algorithm, 0.4};
  constexpr double min_jet_pt = 50;

};

int main(int argn, char** argv) {
  if(argn != 2){
    std::cerr << "Usage: test_psp eventfile";
    return EXIT_FAILURE;
  }

  HEJ::istream in{argv[1]};
  LHEF::Reader reader{in};
  LHEF::Writer writer{std::cerr};
  writer.heprup = reader.heprup;


  HEJ::PhaseSpacePointConfig conf;
  conf.jet_param = HEJ::JetParameters{jet_def, min_jet_pt};
  conf.min_extparton_pt = extpartonptmin;
  conf.max_ext_soft_pt_fraction = max_ext_soft_pt_fraction;

  HEJ::Ranlux64 ran{};

  while(reader.readEvent()){
    const HEJ::Event ev{
      HEJ::UnclusteredEvent{reader.hepeup},
      jet_def, min_jet_pt
    };
    for(int trial = 0; trial < max_trials; ++trial){
      HEJ::PhaseSpacePoint psp{ev, conf, ran};
      if(psp.weight() != 0){
        HEJ::UnclusteredEvent tmp_ev;
        tmp_ev.incoming = psp.incoming();
        tmp_ev.outgoing = psp.outgoing();
        tmp_ev.central = {0,0,0};
        HEJ::Event out_ev{
          std::move(tmp_ev),
          jet_def, min_jet_pt
        };
        if(out_ev.type() != ev.type()){
          using HEJ::event_type::names;
          std::cerr << "Wrong class of phase space point:\n"
            "original event of class " << names[ev.type()] << ":\n";
          writer.hepeup = reader.hepeup;
          writer.writeEvent();
          std::cerr << "Phase space point of class " << names[out_ev.type()] << ":\n";
          writer.hepeup = to_HEPEUP(out_ev, &writer.heprup);
          writer.writeEvent();
          return EXIT_FAILURE;
        }
      }
    }
  }

}
